/*
 * Copyright 2012 Mark Injerd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.pilot51.voicenotify;

import sparta.checkers.quals.*;
import static sparta.checkers.quals.FlowPermission.*;

public class App {
	private @Source({LITERAL, DATABASE, SHARED_PREFERENCES}) String packageName, label;
	private @Source({LITERAL, DATABASE, SHARED_PREFERENCES}) boolean enabled;
	
	App(@Source({LITERAL, DATABASE, SHARED_PREFERENCES}) String pkg,
            @Source({LITERAL, DATABASE, SHARED_PREFERENCES}) String name,
            @Source({LITERAL, DATABASE, SHARED_PREFERENCES}) boolean enable) {
		packageName = pkg;
		label = name;
		enabled = enable;
	}
	
	/**
	 * Updates self in database.
	 * @return This instance.
	 */
	@Source(LITERAL) App updateDb(@Source(LITERAL) App this) {
		Database.addOrUpdateApp(this);
		return this;
	}
	
	void setEnabled(@Source({LITERAL, DATABASE, SHARED_PREFERENCES}) boolean enable, @Sink(CONDITIONAL) boolean updateDb) {
		enabled = enable;
		if (updateDb) Database.updateAppEnable(this);
	}
	
	/** Removes self from database. */
	void remove() {
		Database.removeApp(this);
	}
	
	@Source({LITERAL, DATABASE, SHARED_PREFERENCES}) String getLabel(
            @PolySource @PolySink App this) {
		return label;
	}
	
	@Source({LITERAL, DATABASE, SHARED_PREFERENCES}) String getPackage(
            @PolySource @PolySink App this) {
		return packageName;
	}
	
	@Source({LITERAL, DATABASE, SHARED_PREFERENCES}) boolean getEnabled(
            @PolySource @PolySink App this) {
		return enabled;
	}
}
